
#include <memory>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>

#ifndef BOSK_SDL2
#define BOSK_SDL2

namespace sdl2 {
struct SDL_Deleter {
  void operator()(SDL_Surface*  ptr) { if (ptr) SDL_FreeSurface(ptr); }
  void operator()(SDL_Texture*  ptr) { if (ptr) SDL_DestroyTexture(ptr); }
  void operator()(SDL_Renderer* ptr) { if (ptr) SDL_DestroyRenderer(ptr); }
  void operator()(SDL_Window*   ptr) { if (ptr) SDL_DestroyWindow(ptr); }
  void operator()(SDL_RWops*    ptr) { if (ptr) SDL_RWclose(ptr); }
};

// Unique Pointers
using SDLSurface  = std::unique_ptr<SDL_Surface,  SDL_Deleter>;
using SDLTexture  = std::unique_ptr<SDL_Texture,  SDL_Deleter>;
using SDLRenderer = std::unique_ptr<SDL_Renderer, SDL_Deleter>;
using SDLWindow   = std::unique_ptr<SDL_Window,   SDL_Deleter>;
using SDLRWops    = std::unique_ptr<SDL_RWops,    SDL_Deleter>;
}

#endif // BOSK_SDL2
