
// CAMERA
//===================================

#ifndef CAMERA_H
#define CAMERA_H


// FORWARD DECLARATIONS
//===================================

    class Sprite;


// DEPENDENCIES
//===================================

    #include "direction.h"
    #include "unit.h"
    #include <SDL2/SDL.h>


// CLASS
//===================================

    class Camera
    {
        public:
            //static const int BORDER_PADDING = 2;
            static const int BORDER_PADDING = 0; // DEBUG

            Camera
            (
                int x = 0,
                int y = 0,
                int width = Unit::WINDOW_WIDTH_BLOCKS,
                int height = Unit::WINDOW_HEIGHT_BLOCKS - 2
            );
            ~Camera();

            int x_                     = -1;
            int y_                     = -1;

            int widthBlocks()        const;
            int heightBlocks()       const;
            int widthPixels()        const;
            int heightPixels()       const;
            int x()                  const;
            int y()                  const;
            int prevX()              const;
            int prevY()              const;
            int boundaryTop()        const;
            int boundaryBottom()     const;
            int boundaryLeft()       const;
            int boundaryRight()      const;
            int right()              const;
            int bottom()             const;


            bool offscreenTop( SDL_Rect& r, int padding = 0 ) const;
            bool offscreenRight( SDL_Rect& r, int padding = 0 ) const;
            bool offscreenBottom( SDL_Rect& r, int padding = 0 ) const;
            bool offscreenLeft( SDL_Rect& r, int padding = 0 ) const;
            bool offscreenPartwayTop( SDL_Rect& r ) const;
            bool offscreenPartwayRight( SDL_Rect& r ) const;
            bool offscreenPartwayBottom( SDL_Rect& r ) const;
            bool offscreenPartwayLeft( SDL_Rect& r ) const;
            bool onscreen( SDL_Rect& r, int padding = 0 )  const;
            bool offscreen( SDL_Rect& r, int padding = 0, Direction::Simple direction = Direction::Simple::__NULL ) const;
            int relativeX( SDL_Rect& r ) const;
            int relativeY( SDL_Rect& r ) const;

            bool changed();

            void adjust( Sprite& o, int map_width, int map_height );
            void setPrev();
            void moveDown();
            void moveUp();
            void moveLeft();
            void moveRight();

        private:
            const int width_                 = -1;
            const int height_                = -1;
            int prev_x_                = -1;
            int prev_y_                = -1;
            bool changed_ = true;

            void contain( int map_width, int map_height );
    };

#endif // CAMERA_H
