

// DEPENDENCIES
//===================================

    #include "game.h"
    #include "title_state.h"
    #include <SDL2/SDL.h>
    #include <iostream>


// STATIC PROPERTIES
//===================================


// METHODS
//===================================

    Game::Game()
    {
        if ( !initSDL() )
            running_ = false;

        graphics_.reset( new Graphics() );
        firstState();
    };

    Game::~Game()
    {
        SDL_Quit();
    };

    bool Game::running() const
    {
        return running_;
    };

    void Game::execute()
    {
        SDL_Event event;

        while ( running_ )
        {
            while ( SDL_PollEvent( &event ) != 0 )
            {
                if ( event.type == SDL_QUIT )
                {
                    quit();
                }
                if ( event.type == SDL_KEYDOWN )
                {
                    input_.keyPress( SDL_GetKeyFromScancode( event.key.keysym.scancode ) );
                    input_.keyHold( SDL_GetKeyFromScancode( event.key.keysym.scancode ) );
                }
                if ( event.type == SDL_KEYUP )
                {
                    input_.keyRelease( SDL_GetKeyFromScancode( event.key.keysym.scancode ) );
                }
            }

            if ( SDL_GetTicks() - ticks_ >= fpsMilliseconds() )
            {

                if ( input_.pressed( Input::Action::ESCAPE ) )
                {
                    running_ = false;
                }

                states_.back()->update( *this, input_, *graphics_ );
                render();
                ticks_ = SDL_GetTicks();
                input_.update();
            }
        }
    };

    void Game::render()
    {
        graphics_->clearScreen();
        graphics_->colorCanvas();
        for ( auto &st : states_ )
        {
            st->render( *graphics_ );
        }
        graphics_->presentScreen();
    };

    void Game::quit()
    {
        running_ = false;
    };

    bool Game::initSDL()
    {
        if ( SDL_Init( SDL_INIT_VIDEO ) != 0 )
        {
            SDL_Log( "SDL_Initialization failed: %s", SDL_GetError() );
            return false;
        }

        return true;
    };

    void Game::changeState( std::unique_ptr<GameState> state )
    {
        states_.clear();
        pushState( move( state ) );
    };

    void Game::pushState  ( std::unique_ptr<GameState> state )
    {
        states_.push_back( move(state) );
        //graphics_->newPalette( states_.back()->palette() );
        states_.back()->changePalette( *graphics_ );
        states_.back()->init( *this, *graphics_ );
    };

    void Game::popState()
    {
        if ( !states_.empty() )
        {
            states_.pop_back();
        }
        states_.back()->backFromPop( *this, *graphics_ );
        states_.back()->changePalette( *graphics_ );
    };

    int Game::fpsMilliseconds() const
    {
        return floor( 1000 / FPS );
    };

    void Game::firstState()
    {
        //Ensure state list is clear:
        states_.clear();

        pushState( std::unique_ptr<GameState> ( new TitleState() ) );
    };
