
// Name
//===================================
//
// Graphics
//

#ifndef GRAPHICS_H
#define GRAPHICS_H


// FORWARD DECLARATIONS
//===================================


// DEPENDENCIES
//===================================

    #include "bosk_sdl2.h"
    #include <iostream>
    #include "palette.h"
    #include <SDL2/SDL.h>


// CLASS
//===================================

    class Graphics
    {
        public:
            enum SpriteSheet
            {
                CHARSET,
                BOSKO_LOGO,
                LVTILESET_UNIVERSAL,
                LVTILESET_CITY,
                LVSPRITES_AUTUMN,
                LIMIT
            };

            Graphics();
            ~Graphics();
            const std::string imgAddress( const std::string& relative_path ) const;
            SDL_Texture* loadTexture( const std::string& address ) const;

            void colorCanvas();
            void clearScreen();
            void presentScreen();
            void clearTextures();
            void renderObject( SpriteSheet sheet, SDL_Rect source, SDL_Rect dest, SDL_RendererFlip flip = SDL_FLIP_NONE );
            void renderRect( SDL_Rect box, Uint8 r = 0, Uint8 g = 0, Uint8 b = 0, Uint8 a = 255 );

            void newPalette( Palette::PaletteSet palette );
            void reloadTextures();

        private:
            const char* graphic_addresses_[ SpriteSheet::LIMIT ] =
            {
                "charset.png",
                "bosko_logo.png",
                "tileset_level_universal.png",
                "tileset_level_city.png",
                "sprites_level_autumn.png"
            };
            sdl2::SDLTexture textures_[SpriteSheet::LIMIT] = { nullptr };
            sdl2::SDLRenderer renderer_ = nullptr; // Special init & destruct ptr.
            sdl2::SDLWindow   window_ = nullptr; // Special init & destruct ptr.
            SDL_Rect screen_;
            Palette* palette_ = new Palette();

            int magnification_ = 1;
            int left_edge_ = 0;
            int top_edge_ = 0;

            int magnified( int n ) const;
            void adjustMagnification();
            void adjustScreen();

            bool createRenderer();
            bool createWindow();

            SDL_Rect sourceRelativeToScreen( SDL_Rect source ) const;
    };

#endif // GRAPHICS_H

