
// Name
//===================================
//
// Map
//


// DEPENDENCIES
//===================================

    #include <algorithm>
    #include <iostream>
    #include "map.h"
    #include "unit.h"


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    Map::Map
    (
        std::vector<int> blocks,
        int width,
        int height,
        Palette::PaletteSet palette
    )
    :
        blocks_ ( blocks ),
        width_ ( width ),
        height_ ( height ),
        palette_ ( palette )
    {
    };

    Map::~Map() {};

    int Map::widthBlocks() const
    {
        return width_;
    };

    int Map::heightBlocks() const
    {
        return height_;
    };

    int Map::widthPixels() const
    {
        return Unit::BlocksToPixels( width_ );
    };

    int Map::heightPixels() const
    {
        return Unit::BlocksToPixels( height_ );
    };

    int Map::blocksSize() const
    {
        return std::min( widthBlocks() * heightBlocks(), (int)blocks_.size() );
    };

    int Map::block( int n ) const
    {
        return blocks_[ n ];
    };

    int Map::mapX( int n ) const
    {
        return n % widthBlocks();
    };

    int Map::mapY( int n ) const
    {
        return floor( n / widthBlocks() );
    };

    const bool Map::changed() const
    {
        return changed_;
    };

    void Map::changeBlock( int where, int value )
    {
        if ( inBounds( where ) )
        {
            blocks_[ where ] = value;
        }
        changed_ = true;
    };

    void Map::deleteBlock( int where )
    {
        changeBlock( where, 0 );
    };

    bool Map::inBounds( int n ) const
    {
        return n > 0 && n < blocks_.size();
    };

    void Map::update()
    {
        changed_ = false;
    };

    Palette::PaletteSet Map::palette() const
    {
        return palette_;
    };
