



// Name
//===================================
//
// Text
//

#ifndef TEXT_H
#define TEXT_H


// FORWARD DECLARATIONS
//===================================

    class Camera;
    class Graphics;


// DEPENDENCIES
//===================================

    #include "counter.h"
    #include <map>
    #include <SDL2/SDL.h>
    #include <string>
    #include "timers/timer_repeat.h"


// CLASS
//===================================

    class Text
    {
        public:
            enum CharSymbols
            {
                PTS = 0,
                HEART = 1,
                RIGHT_ARROW = 2,
                LEFT_ARROW = 3,
                DOWN_ARROW = 4,
                UP_ARROW = 5
            };

            enum FontShade
            {
                BLACK = 0,
                DARK_GRAY = 1,
                DARK_MID_GRAY = 2,
                LIGHT_MID_GRAY = 3,
                LIGHT_GRAY = 4,
                WHITE = 5,
                __NULL = 6
            };

            enum FontAlign
            {
                FA_LEFT,
                FA_CENTER,
                FA_RIGHT
            };

            Text
            (
                std::string words,
                int x = 0,
                int y = 0,
                FontShade shade = FontShade::BLACK,
                FontAlign align = FontAlign::FA_LEFT,
                bool blink = false
            );
            ~Text();
            void update();
            void render( Graphics& graphics, Camera* camera = nullptr, FontShade shade = FontShade::__NULL );
            void renderNumber( Graphics& graphics, int n, int digits = -1, Camera* camera = nullptr, FontShade shade = FontShade::BLACK );

        private:
            const int NUM_OF_FONT_SHADES = 6;
            const int CHARSET_WIDTH_MINI_BLOCKS = 32;
            static std::map<char, int> char_conversion_;
            std::string words_;
            int x_ = 0;
            int y_ = 0;
            FontShade shade_ = FontShade::BLACK;
            FontAlign align_ = FontAlign::FA_LEFT;
            bool blink_ = false;
            SDL_Rect rect_ = { 0, 0, 8, 8 };
            Counter blink_counter_ = Counter( 0, NUM_OF_FONT_SHADES-1, 0, true );
            TimerRepeat blink_timer_;
            Direction::Vertical blink_dir_ = Direction::Vertical::UP;

            int getDigit( int n, int d, int remain = 10 ) const;
            int numODigits( int n ) const;
            int frameX( int n ) const;
            int frameY( int n, FontShade shade ) const;
            int shadeOffset( FontShade shade ) const;
            int shadeNum() const;
            int centerX() const;
    };

#endif // TEXT_H

